import datetime

from odoo.addons.saas_worker import metabase


class EmailLimitRules:
    def __init__(self, env):
        self.dbname = env.cr.dbname
        self.env = env
        self.meta_db = metabase.get(self.dbname)

    def evaluate(self):
        results = []
        for rule in dir(self):
            if rule.startswith('rule_'):
                f = getattr(self, rule)
                result = f()
                results.append(result)
        return results

    @classmethod
    def select_truthy(cls, results):
        truthy_rules = []
        for r in results:
            if r['truthy']:
                r_copy = dict(**r)
                del r_copy['truthy']
                truthy_rules.append(r_copy)

        return truthy_rules

    def compute_bounce_ratio(self):
        total_mail_count = self.meta_db.email_total_count + self.meta_db.email_daily_count + self.meta_db.ext_email_daily_count
        if total_mail_count < 500:
            # ignore bounce ratio for small email volume
            return None

        return (self.meta_db.bounce_total_count + self.meta_db.bounce_daily_count) / total_mail_count

    def compute_is_one_app_free(self):
        db = self.meta_db
        return db.mode == 'trial' and not db.extra_apps

    #######################
    # Here come the rules #
    #######################
    def rule_adult_database(self):
        """
        Compare the creation date with the current date
        Truthy if the difference is large enough
        """
        age_threshold = datetime.datetime.utcnow() - datetime.timedelta(days=365.24 / 2)
        result = self.meta_db.create_date <= age_threshold

        return {
            'grade': 5,
            'text': 'Database is old enough',
            'truthy': result,
        }

    def rule_bad_bounce(self):
        """
        Truthy if the database has a bad bounce ratio
        """

        bounce_ratio = self.compute_bounce_ratio()
        result = bounce_ratio is not None and bounce_ratio >= 0.04

        return {
            'grade': -5,
            'text': 'Bad bounce ratio' + (' (%0.2f\xa0%%)' % (100 * bounce_ratio) if bounce_ratio is not None else ''),
            'truthy': result,
        }

    def rule_good_bounce(self):
        """
        Truthy if the database has a good bounce ratio
        """

        bounce_ratio = self.compute_bounce_ratio()
        result = bounce_ratio is not None and bounce_ratio <= 0.0015

        return {
            'grade': 5,
            'text': 'Good bounce ratio' + (' (%0.2f\xa0%%)' % (100 * bounce_ratio) if bounce_ratio is not None else ''),
            'truthy': result,
        }

    def rule_limit_increase_throttle(self):
        """
        Truthy if the email limit was already increased too recently
        """
        history_limit = (datetime.date.today() - datetime.timedelta(days=15)).strftime('%Y-%m-%d')
        rows = metabase.get_stats(self.dbname, date_from=history_limit)
        history = sorted(((row.date, row.email_limit) for row in rows), reverse=True)
        current_email_limit = self.meta_db.email_daily_limit

        result = False
        for date, email_limit in history:
            if email_limit < current_email_limit:
                result = True
                break

        return {
            'grade': -2,
            'text': 'Email limit recently increased',
            'truthy': result,
        }

    def rule_one_app_free(self):
        """
        Truthy if the database only has one application installed
        """
        result = self.compute_is_one_app_free()

        return {
            'grade': -5,
            'text': 'One-app-free database',
            'truthy': result,
        }

    def rule_only_mass_mailing(self):
        is_one_app_free = self.compute_is_one_app_free()
        if is_one_app_free:
            installed_apps = self.env['ir.module.module'].search_read(domain=[
                ('state', '=', 'installed'),
                ('application', '=', True),
            ], fields=['name'])
            result = any(i['name'] == 'mass_mailing' for i in installed_apps)
        else:
            result = False

        return {
            'grade': -3,
            'text': 'Only mass mailing is installed',
            'truthy': result,
        }

    def rule_paid_db(self):
        """
        Truthy if the database mode is 'paid'
        """
        result = self.meta_db.mode == 'paid'

        return {
            'grade': 3,
            'text': 'Paid database',
            'truthy': result,
        }

    def rule_very_bad_bounce(self):
        """
        Truthy if the database has a very bad bounce ratio
        """

        bounce_ratio = self.compute_bounce_ratio()
        result = bounce_ratio is not None and bounce_ratio >= 0.1

        return {
            'grade': -3,
            'text': 'Very bad bounce ratio' + (' (%0.2f\xa0%%)' % (100 * bounce_ratio) if bounce_ratio is not None else ''),
            'truthy': result,
        }

    def rule_very_good_bounce(self):
        """
        Truthy if the database has a very good bounce ratio
        """

        bounce_ratio = self.compute_bounce_ratio()
        result = bounce_ratio is not None and bounce_ratio == 0

        return {
            'grade': 3,
            'text': 'Very good bounce ratio' + (' (%0.2f\xa0%%)' % (100 * bounce_ratio) if bounce_ratio is not None else ''),
            'truthy': result,
        }

    def rule_with_mass_mailing(self):
        mass_mailing = self.env['ir.module.module'].search([('name', '=', 'mass_mailing')])
        result = mass_mailing.state == 'installed'

        return {
            'grade': -3,
            'text': 'Mass mailing is installed',
            'truthy': result,
        }
